﻿Imports RMCLinkNET

Public Class Form1

    Dim xPosLim, xNegLim, yPosLim, yNegLim
    Dim lastPosX() As Single = New Single(100) {}
    Dim lastPosY() As Single = New Single(100) {}
    Dim posIndex As Integer
    Dim trailLen As Integer
    'Dim imgDot As Bitmap
    Dim startupTrailCntr As Integer = 0
    Dim dotWidth As Single = 6

    Private rmc As RMCLink

    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

        ' Since we are disconnected to start, enable and disable buttons and labels accordingly.
        connectButton.Enabled = True
        disconnectButton.Enabled = False

        posLimXValue.Visible = False
        negLimXValue.Visible = False
        posLimYValue.Visible = False
        negLimYValue.Visible = False

    End Sub

    Private Sub Form1_Closing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles MyBase.FormClosing
        If rmc IsNot Nothing Then
            If rmc.IsConnected(PingType.DoNotPing) Then
                rmc.Disconnect()
            End If
        End If
    End Sub

    Private Sub connectButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles connectButton.Click
        Try
            ' Create an RMCLink object.
            ' "rmc70.deltamotion.com" is the DNS name for an RMC70 provided by Delta 
            ' that is always available through the web. 
            ' Typically, an IP address such as "192.168.0.161" would be used.
            If rmcType.Text = "RMC70" Then
                rmc = RMCLink.CreateEthernetLink(DeviceType.RMC70, IPAddrValue.Text)
            Else
                rmc = RMCLink.CreateEthernetLink(DeviceType.RMC150, IPAddrValue.Text)
            End If

            rmc.Connect()

            ' We have successfully connected, so update which buttons are enabled.
            disconnectButton.Enabled = True
            connectButton.Enabled = False
            rmcType.Enabled = False
            IPAddrValue.Enabled = False
            axisSelX.Enabled = False
            axisSelY.Enabled = False

            ' Read axis travel limits.
            ReadTravLimits()
            Form1_Resize(Nothing, Nothing)

            'Make travel limits visible
            posLimXValue.Visible = True
            negLimXValue.Visible = True
            posLimYValue.Visible = True
            negLimYValue.Visible = True

            trailLen = TrailLenBar.Value

            ' Start Timer
            Timer1.Start()

        Catch ex As ConnectionNotMadeException
            MessageBox.Show("Unable to connect. " + ex.Message)
        End Try
    End Sub

    Private Sub disconnectButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles disconnectButton.Click
        rmc.Disconnect()

        ' We have successfully disconnected, so update which buttons are enabled.
        connectButton.Enabled = True
        disconnectButton.Enabled = False
        startupTrailCntr = 0
        rmcType.Enabled = True
        IPAddrValue.Enabled = True
        axisSelX.Enabled = True
        axisSelY.Enabled = True

        'Clear display
        Dim Gr As Drawing.Graphics
        Gr = xyPlotBox.CreateGraphics
        Gr.Clear(Color.White)

        ' Stop Timer
        Timer1.Stop()

    End Sub

    Private Sub Timer1_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Timer1.Tick

        If Me.WindowState <> FormWindowState.Minimized Then
            Try
                Dim dataX() As Single = New Single(1) {}
                Dim dataY() As Single = New Single(1) {}

                ' Read the Actual Position (F8+n:8).
                rmc.ReadFFile(FileNumber70.fn70StatusAxis0 + axisSelX.Value, 8, dataX, 0, 1)
                rmc.ReadFFile(FileNumber70.fn70StatusAxis0 + axisSelY.Value, 8, dataY, 0, 1)

                posIndex = (posIndex + 1) Mod lastPosX.Length
                If startupTrailCntr < (lastPosX.Length + 1) Then
                    startupTrailCntr = startupTrailCntr + 1
                End If
                lastPosX(posIndex) = dataX(0)
                lastPosY(posIndex) = dataY(0)

                drawXYPath()

            Catch ex As ReadWriteFailedException
                disconnectButton_Click(Nothing, Nothing)
                MessageBox.Show("Unable to read data. " + ex.Message)
                Timer1.Stop()
            End Try
        End If

    End Sub

    Private Sub ReadTravLimits()
        Try
            Dim dataX() As Single = New Single(1) {}
            Dim dataY() As Single = New Single(1) {}
            Dim axisParamFile

            If rmcType.Text = "RMC70" Then
                axisParamFile = 12
            Else
                axisParamFile = 24
            End If

            ' Read the travel limits (elements 92 and 93).
            rmc.ReadFFile(axisParamFile + axisSelX.Value, 92, dataX, 0, 2)
            rmc.ReadFFile(axisParamFile + axisSelY.Value, 92, dataY, 0, 2)

            ' Update the display values.
            posLimXValue.Text = String.Format("{0:F1}", dataX(0))
            negLimXValue.Text = String.Format("{0:F1}", dataX(1))
            posLimYValue.Text = String.Format("{0:F1}", dataY(0))
            negLimYValue.Text = String.Format("{0:F1}", dataY(1))

            xPosLim = dataX(0)
            xNegLim = dataX(1)
            yPosLim = dataY(0)
            yNegLim = dataY(1)

        Catch ex As ReadWriteFailedException
            disconnectButton_Click(Nothing, Nothing)
            MessageBox.Show("Unable to read data. " + ex.Message)
            Timer1.Stop()
        End Try
    End Sub

    Private Sub drawXYPath()

        Dim PreBitmap As New Drawing.Bitmap(xyPlotBox.Size.Width, xyPlotBox.Size.Height)

        Dim PreGr As Drawing.Graphics
        PreGr = Graphics.FromImage(PreBitmap)
        PreGr.Clear(Color.White)

        Dim scaleX = xyPlotBox.Size.Width / Math.Abs(xPosLim - xNegLim)
        Dim scaleY = xyPlotBox.Size.Height / Math.Abs(yPosLim - yNegLim)

        Dim loopStart, loopEnd
        loopStart = (posIndex - Math.Min(startupTrailCntr, trailLen) + 1) Mod lastPosX.Length
        If loopStart < 0 Then
            loopStart = lastPosX.Length + loopStart
        End If
        loopEnd = loopStart + Math.Min(startupTrailCntr, trailLen) - 2

        For j = loopStart To loopEnd
            Dim alpha = ((j - loopStart) / (trailLen + 1)) * 255

            Dim thePen = New Pen(Color.FromArgb(alpha, 0, 0, 0))
            'thePen = Pens.Black
            Dim thisX = lastPosX(j Mod lastPosX.Length)
            Dim thisY = lastPosY(j Mod lastPosX.Length)
            Dim nextX = lastPosX((j + 1) Mod lastPosX.Length)
            Dim nextY = lastPosY((j + 1) Mod lastPosX.Length)

            PreGr.DrawLine(thePen, (thisX - xNegLim) * scaleX, (thisY - yNegLim) * -scaleY + xyPlotBox.Height, (nextX - xNegLim) * scaleX, (nextY - yNegLim) * -scaleY + xyPlotBox.Height)
        Next j
        PreGr.FillEllipse(Brushes.Crimson, (lastPosX(posIndex) - xNegLim) * scaleX - dotWidth / 2, (lastPosY(posIndex) - yNegLim) * -scaleY + xyPlotBox.Height - dotWidth / 2, dotWidth, dotWidth)

        PreGr.Dispose()

        'Copy drawing to screen
        Dim RealGr As Drawing.Graphics
        RealGr = xyPlotBox.CreateGraphics
        RealGr.SmoothingMode = Drawing2D.SmoothingMode.None
        RealGr.DrawImageUnscaled(PreBitmap, 0, 0, xyPlotBox.Size.Width, xyPlotBox.Size.Height)
        RealGr.Dispose()

    End Sub

    Private Sub TrailLenBar_Scroll(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TrailLenBar.Scroll
        trailLen = TrailLenBar.Value
    End Sub

    Private Sub Form1_Resize(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Resize
        Dim horzSpace, vertSpace, vertPU, horzPU As Double
        Dim ratioSpace, ratioPU As Double                    'ratio is y over x

        'Location of xyplotBox is 165, 20, with 20 padding on right and bottom.

        horzSpace = Me.ClientSize.Width - 185
        vertSpace = Me.ClientSize.Height - 40

        If xPosLim <> Nothing Then
            ratioSpace = vertSpace / horzSpace
            horzPU = xPosLim - xNegLim
            vertPU = yPosLim - yNegLim
            ratioPU = vertPU / horzPU

            If ratioPU <= ratioSpace Then
                xyPlotBox.Width = horzSpace
                xyPlotBox.Height = ratioPU * xyPlotBox.Width
            Else
                xyPlotBox.Height = vertSpace
                xyPlotBox.Width = xyPlotBox.Height / ratioPU
            End If
        Else
            xyPlotBox.Width = horzSpace
            xyPlotBox.Height = vertSpace
        End If

        'place axis labels
        negLimXValue.Location = New Point(xyPlotBox.Location.X - 10, xyPlotBox.Location.Y + xyPlotBox.Height + 3)
        posLimXValue.Location = New Point(xyPlotBox.Location.X + xyPlotBox.Width - 10, xyPlotBox.Location.Y + xyPlotBox.Height + 3)
        posLimYValue.Location = New Point(xyPlotBox.Location.X - 30, xyPlotBox.Location.Y)
        negLimYValue.Location = New Point(xyPlotBox.Location.X - 30, xyPlotBox.Location.Y + xyPlotBox.Height - 10)

    End Sub


End Class
